@@ -1,22 +1,36 @@
-require 'action_view/helpers/tag_helper'
-
-begin
-  require 'html/document'
-rescue LoadError
-  html_scanner_path = "#{File.dirname(__FILE__)}/../../action_controller/vendor/html-scanner"
-  if File.directory?(html_scanner_path)
-    $:.unshift html_scanner_path
-    require 'html/document'
-  end
-end
+require 'active_support/core_ext/object/blank'
+require 'active_support/core_ext/string/filters'
 
 module ActionView
+  # = Action View Text Helpers
   module Helpers #:nodoc:
     # The TextHelper module provides a set of methods for filtering, formatting
     # and transforming strings, which can reduce the amount of inline Ruby code in
-    # your views. These helper methods extend ActionView making them callable
+    # your views. These helper methods extend Action View making them callable
     # within your template files.
+    #
+    # ==== Sanitization
+    #
+    # Most text helpers by default sanitize the given content, but do not escape it.
+    # This means HTML tags will appear in the page but all malicious code will be removed.
+    # Let's look at some examples using the +simple_format+ method:
+    #
+    #   simple_format('<a href="http://example.com/">Example</a>')
+    #   # => "<p><a href=\"http://example.com/\">Example</a></p>"
+    #
+    #   simple_format('<a href="javascript:alert(\'no!\')">Example</a>')
+    #   # => "<p><a>Example</a></p>"
+    #
+    # If you want to escape all content, you should invoke the +h+ method before
+    # calling the text helper.
+    #
+    #   simple_format h('<a href="http://example.com/">Example</a>')
+    #   # => "<p>&lt;a href=\"http://example.com/\"&gt;Example&lt;/a&gt;</p>"
     module TextHelper
+      extend ActiveSupport::Concern
+
+      include SanitizeHelper
+      include TagHelper
       # The preferred method of outputting text in your views is to use the
       # <%= "text" %> eRuby syntax. The regular _puts_ and _print_ methods
       # do not operate as expected in an eRuby code block. If you absolutely must
@@ -27,62 +41,50 @@ module ActionView
       #       concat "hello"
       #       # is the equivalent of <%= "hello" %>
       #
-      #       if (logged_in == true):
+      #       if logged_in
       #         concat "Logged in!"
       #       else
       #         concat link_to('login', :action => login)
       #       end
       #       # will either display "Logged in!" or a login link
       #   %>
-      def concat(string, unused_binding = nil)
-        if unused_binding
-          ActiveSupport::Deprecation.warn("The binding argument of #concat is no longer needed.  Please remove it from your views and helpers.", caller)
-        end
-
+      def concat(string)
         output_buffer << string
       end
 
+      def safe_concat(string)
+        output_buffer.respond_to?(:safe_concat) ? output_buffer.safe_concat(string) : concat(string)
+      end
+
       # Truncates a given +text+ after a given <tt>:length</tt> if +text+ is longer than <tt>:length</tt>
-      # (defaults to 30). The last characters will be replaced with the <tt>:omission</tt> (defaults to "...").
+      # (defaults to 30). The last characters will be replaced with the <tt>:omission</tt> (defaults to "...")
+      # for a total length not exceeding <tt>:length</tt>.
+      #
+      # Pass a <tt>:separator</tt> to truncate +text+ at a natural break.
+      #
+      # The result is not marked as HTML-safe, so will be subject to the default escaping when
+      # used in views, unless wrapped by <tt>raw()</tt>. Care should be taken if +text+ contains HTML tags
+      # or entities, because truncation may produce invalid HTML (such as unbalanced or incomplete tags).
       #
       # ==== Examples
       #
       #   truncate("Once upon a time in a world far far away")
-      #   # => Once upon a time in a world f...
+      #   # => "Once upon a time in a world..."
       #
-      #   truncate("Once upon a time in a world far far away", :length => 14)
-      #   # => Once upon a...
+      #   truncate("Once upon a time in a world far far away", :length => 17)
+      #   # => "Once upon a ti..."
       #
-      #   truncate("And they found that many people were sleeping better.", :length => 25, "(clipped)")
-      #   # => And they found that many (clipped)
+      #   truncate("Once upon a time in a world far far away", :length => 17, :separator => ' ')
+      #   # => "Once upon a..."
       #
-      #   truncate("And they found that many people were sleeping better.", :omission => "... (continued)", :length => 15)
-      #   # => And they found... (continued)
+      #   truncate("And they found that many people were sleeping better.", :length => 25, :omission => '... (continued)')
+      #   # => "And they f... (continued)"
       #
-      # You can still use <tt>truncate</tt> with the old API that accepts the
-      # +length+ as its optional second and the +ellipsis+ as its
-      # optional third parameter:
-      #   truncate("Once upon a time in a world far far away", 14)
-      #   # => Once upon a time in a world f...
-      #
-      #   truncate("And they found that many people were sleeping better.", 15, "... (continued)")
-      #   # => And they found... (continued)
-      def truncate(text, *args)
-        options = args.extract_options!
-        unless args.empty?
-          ActiveSupport::Deprecation.warn('truncate takes an option hash instead of separate ' +
-            'length and omission arguments', caller)
-
-          options[:length] = args[0] || 30
-          options[:omission] = args[1] || "..."
-        end
-        options.reverse_merge!(:length => 30, :omission => "...")
-
-        if text
-          l = options[:length] - options[:omission].mb_chars.length
-          chars = text.mb_chars
-          (chars.length > options[:length] ? chars[0...l] + options[:omission] : text).to_s
-        end
+      #   truncate("<p>Once upon a time in a world far far away</p>")
+      #   # => "<p>Once upon a time in a wo..."
+      def truncate(text, options = {})
+        options.reverse_merge!(:length => 30)
+        text.truncate(options.delete(:length), options) if text
       end
 
       # Highlights one or more +phrases+ everywhere in +text+ by inserting it into
@@ -109,16 +111,20 @@ module ActionView
       def highlight(text, phrases, *args)
         options = args.extract_options!
         unless args.empty?
+          ActiveSupport::Deprecation.warn "Calling highlight with a highlighter as an argument is deprecated. " \
+          "Please call with :highlighter => '#{args[0]}' instead.", caller
+
           options[:highlighter] = args[0] || '<strong class="highlight">\1</strong>'
         end
         options.reverse_merge!(:highlighter => '<strong class="highlight">\1</strong>')
 
+        text = sanitize(text) unless options[:sanitize] == false
         if text.blank? || phrases.blank?
           text
         else
           match = Array(phrases).map { |p| Regexp.escape(p) }.join('|')
-          text.gsub(/(#{match})/i, options[:highlighter])
-        end
+          text.gsub(/(#{match})(?![^<]*?>)/i, options[:highlighter])
+        end.html_safe
       end
 
       # Extracts an excerpt from +text+ that matches the first instance of +phrase+.
@@ -149,28 +155,28 @@ module ActionView
       #   excerpt('This is an example', 'an', 5)                   # => ...s is an exam...
       #   excerpt('This is also an example', 'an', 8, '<chop> ')   # => <chop> is also an example
       def excerpt(text, phrase, *args)
+        return unless text && phrase
+
         options = args.extract_options!
         unless args.empty?
+          ActiveSupport::Deprecation.warn "Calling excerpt with radius and omission as arguments is deprecated. " \
+          "Please call with :radius => #{args[0]}#{", :omission => '#{args[1]}'" if args[1]} instead.", caller
+
           options[:radius] = args[0] || 100
           options[:omission] = args[1] || "..."
         end
         options.reverse_merge!(:radius => 100, :omission => "...")
 
-        if text && phrase
-          phrase = Regexp.escape(phrase)
+        phrase = Regexp.escape(phrase)
+        return unless found_pos = text.mb_chars =~ /(#{phrase})/i
 
-          if found_pos = text.mb_chars =~ /(#{phrase})/i
-            start_pos = [ found_pos - options[:radius], 0 ].max
-            end_pos   = [ [ found_pos + phrase.mb_chars.length + options[:radius] - 1, 0].max, text.mb_chars.length ].min
+        start_pos = [ found_pos - options[:radius], 0 ].max
+        end_pos   = [ [ found_pos + phrase.mb_chars.length + options[:radius] - 1, 0].max, text.mb_chars.length ].min
 
-            prefix  = start_pos > 0 ? options[:omission] : ""
-            postfix = end_pos < text.mb_chars.length - 1 ? options[:omission] : ""
+        prefix  = start_pos > 0 ? options[:omission] : ""
+        postfix = end_pos < text.mb_chars.length - 1 ? options[:omission] : ""
 
-            prefix + text.mb_chars[start_pos..end_pos].strip + postfix
-          else
-            nil
-          end
-        end
+        prefix + text.mb_chars[start_pos..end_pos].strip + postfix
       end
 
       # Attempts to pluralize the +singular+ word unless +count+ is 1. If
@@ -190,7 +196,7 @@ module ActionView
       #   pluralize(0, 'person')
       #   # => 0 people
       def pluralize(count, singular, plural = nil)
-        "#{count || 0} " + ((count == 1 || count == '1') ? singular : (plural || singular.pluralize))
+        "#{count || 0} " + ((count == 1 || count =~ /^1(\.0+)?$/) ? singular : (plural || singular.pluralize))
       end
 
       # Wraps the +text+ into lines no longer than +line_width+ width. This method
@@ -217,6 +223,9 @@ module ActionView
       def word_wrap(text, *args)
         options = args.extract_options!
         unless args.blank?
+          ActiveSupport::Deprecation.warn "Calling word_wrap with line_width as an argument is deprecated. " \
+          "Please call with :line_width => #{args[0]} instead.", caller
+
           options[:line_width] = args[0] || 80
         end
         options.reverse_merge!(:line_width => 80)
@@ -226,101 +235,18 @@ module ActionView
         end * "\n"
       end
 
-      begin
-        require_library_or_gem "redcloth" unless Object.const_defined?(:RedCloth)
-
-        # Returns the text with all the Textile[http://www.textism.com/tools/textile] codes turned into HTML tags.
-        #
-        # You can learn more about Textile's syntax at its website[http://www.textism.com/tools/textile].
-        # <i>This method is only available if RedCloth[http://whytheluckystiff.net/ruby/redcloth/]
-        # is available</i>.
-        #
-        # ==== Examples
-        #   textilize("*This is Textile!*  Rejoice!")
-        #   # => "<p><strong>This is Textile!</strong>  Rejoice!</p>"
-        #
-        #   textilize("I _love_ ROR(Ruby on Rails)!")
-        #   # => "<p>I <em>love</em> <acronym title="Ruby on Rails">ROR</acronym>!</p>"
-        #
-        #   textilize("h2. Textile makes markup -easy- simple!")
-        #   # => "<h2>Textile makes markup <del>easy</del> simple!</h2>"
-        #
-        #   textilize("Visit the Rails website "here":http://www.rubyonrails.org/.)
-        #   # => "<p>Visit the Rails website <a href="http://www.rubyonrails.org/">here</a>.</p>"
-        def textilize(text)
-          if text.blank?
-            ""
-          else
-            textilized = RedCloth.new(text, [ :hard_breaks ])
-            textilized.hard_breaks = true if textilized.respond_to?(:hard_breaks=)
-            textilized.to_html
-          end
-        end
-
-        # Returns the text with all the Textile codes turned into HTML tags,
-        # but without the bounding <p> tag that RedCloth adds.
-        #
-        # You can learn more about Textile's syntax at its website[http://www.textism.com/tools/textile].
-        # <i>This method is only available if RedCloth[http://whytheluckystiff.net/ruby/redcloth/]
-        # is available</i>.
-        #
-        # ==== Examples
-        #   textilize_without_paragraph("*This is Textile!*  Rejoice!")
-        #   # => "<strong>This is Textile!</strong>  Rejoice!"
-        #
-        #   textilize_without_paragraph("I _love_ ROR(Ruby on Rails)!")
-        #   # => "I <em>love</em> <acronym title="Ruby on Rails">ROR</acronym>!"
-        #
-        #   textilize_without_paragraph("h2. Textile makes markup -easy- simple!")
-        #   # => "<h2>Textile makes markup <del>easy</del> simple!</h2>"
-        #
-        #   textilize_without_paragraph("Visit the Rails website "here":http://www.rubyonrails.org/.)
-        #   # => "Visit the Rails website <a href="http://www.rubyonrails.org/">here</a>."
-        def textilize_without_paragraph(text)
-          textiled = textilize(text)
-          if textiled[0..2] == "<p>" then textiled = textiled[3..-1] end
-          if textiled[-4..-1] == "</p>" then textiled = textiled[0..-5] end
-          return textiled
-        end
-      rescue LoadError
-        # We can't really help what's not there
-      end
-
-      begin
-        require_library_or_gem "bluecloth" unless Object.const_defined?(:BlueCloth)
-
-        # Returns the text with all the Markdown codes turned into HTML tags.
-        # <i>This method is only available if BlueCloth[http://www.deveiate.org/projects/BlueCloth]
-        # is available</i>.
-        #
-        # ==== Examples
-        #   markdown("We are using __Markdown__ now!")
-        #   # => "<p>We are using <strong>Markdown</strong> now!</p>"
-        #
-        #   markdown("We like to _write_ `code`, not just _read_ it!")
-        #   # => "<p>We like to <em>write</em> <code>code</code>, not just <em>read</em> it!</p>"
-        #
-        #   markdown("The [Markdown website](http://daringfireball.net/projects/markdown/) has more information.")
-        #   # => "<p>The <a href="http://daringfireball.net/projects/markdown/">Markdown website</a>
-        #   #     has more information.</p>"
-        #
-        #   markdown('![The ROR logo](http://rubyonrails.com/images/rails.png "Ruby on Rails")')
-        #   # => '<p><img src="http://rubyonrails.com/images/rails.png" alt="The ROR logo" title="Ruby on Rails" /></p>'
-        def markdown(text)
-          text.blank? ? "" : BlueCloth.new(text).to_html
-        end
-      rescue LoadError
-        # We can't really help what's not there
-      end
-
       # Returns +text+ transformed into HTML using simple formatting rules.
       # Two or more consecutive newlines(<tt>\n\n</tt>) are considered as a
       # paragraph and wrapped in <tt><p></tt> tags. One newline (<tt>\n</tt>) is
       # considered as a linebreak and a <tt><br /></tt> tag is appended. This
       # method does not remove the newlines from the +text+.
       #
-      # You can pass any HTML attributes into <tt>html_options</tt>.  These
+      # You can pass any HTML attributes into <tt>html_options</tt>. These
       # will be added to all created paragraphs.
+      #
+      # ==== Options
+      # * <tt>:sanitize</tt> - If +false+, does not sanitize +text+.
+      #
       # ==== Examples
       #   my_text = "Here is some basic text...\n...with a line break."
       #
@@ -334,73 +260,25 @@ module ActionView
       #
       #   simple_format("Look ma! A class!", :class => 'description')
       #   # => "<p class='description'>Look ma! A class!</p>"
-      def simple_format(text, html_options={})
+      #
+      #   simple_format("<span>I'm allowed!</span> It's true.", {}, :sanitize => false)
+      #   # => "<p><span>I'm allowed!</span> It's true.</p>"
+      def simple_format(text, html_options={}, options={})
+        text = '' if text.nil?
+        text = text.dup
         start_tag = tag('p', html_options, true)
-        text = text.to_s.dup
+        text = sanitize(text) unless options[:sanitize] == false
+        text = text.to_str
         text.gsub!(/\r\n?/, "\n")                    # \r\n and \r -> \n
         text.gsub!(/\n\n+/, "</p>\n\n#{start_tag}")  # 2+ newline  -> paragraph
         text.gsub!(/([^\n]\n)(?=[^\n])/, '\1<br />') # 1 newline   -> br
         text.insert 0, start_tag
-        text << "</p>"
-      end
-
-      # Turns all URLs and e-mail addresses into clickable links. The <tt>:link</tt> option
-      # will limit what should be linked. You can add HTML attributes to the links using
-      # <tt>:href_options</tt>. Possible values for <tt>:link</tt> are <tt>:all</tt> (default),
-      # <tt>:email_addresses</tt>, and <tt>:urls</tt>. If a block is given, each URL and
-      # e-mail address is yielded and the result is used as the link text.
-      #
-      # ==== Examples
-      #   auto_link("Go to http://www.rubyonrails.org and say hello to david@loudthinking.com")
-      #   # => "Go to <a href=\"http://www.rubyonrails.org\">http://www.rubyonrails.org</a> and
-      #   #     say hello to <a href=\"mailto:david@loudthinking.com\">david@loudthinking.com</a>"
-      #
-      #   auto_link("Visit http://www.loudthinking.com/ or e-mail david@loudthinking.com", :link => :urls)
-      #   # => "Visit <a href=\"http://www.loudthinking.com/\">http://www.loudthinking.com/</a>
-      #   #     or e-mail david@loudthinking.com"
-      #
-      #   auto_link("Visit http://www.loudthinking.com/ or e-mail david@loudthinking.com", :link => :email_addresses)
-      #   # => "Visit http://www.loudthinking.com/ or e-mail <a href=\"mailto:david@loudthinking.com\">david@loudthinking.com</a>"
-      #
-      #   post_body = "Welcome to my new blog at http://www.myblog.com/.  Please e-mail me at me@email.com."
-      #   auto_link(post_body, :href_options => { :target => '_blank' }) do |text|
-      #     truncate(text, 15)
-      #   end
-      #   # => "Welcome to my new blog at <a href=\"http://www.myblog.com/\" target=\"_blank\">http://www.m...</a>.
-      #         Please e-mail me at <a href=\"mailto:me@email.com\">me@email.com</a>."
-      #
-      #
-      # You can still use <tt>auto_link</tt> with the old API that accepts the
-      # +link+ as its optional second parameter and the +html_options+ hash
-      # as its optional third parameter:
-      #   post_body = "Welcome to my new blog at http://www.myblog.com/. Please e-mail me at me@email.com."
-      #   auto_link(post_body, :urls)     # => Once upon\na time
-      #   # => "Welcome to my new blog at <a href=\"http://www.myblog.com/\">http://www.myblog.com</a>.
-      #         Please e-mail me at me@email.com."
-      #
-      #   auto_link(post_body, :all, :target => "_blank")     # => Once upon\na time
-      #   # => "Welcome to my new blog at <a href=\"http://www.myblog.com/\" target=\"_blank\">http://www.myblog.com</a>.
-      #         Please e-mail me at <a href=\"mailto:me@email.com\">me@email.com</a>."
-      def auto_link(text, *args, &block)#link = :all, href_options = {}, &block)
-        return '' if text.blank?
-
-        options = args.size == 2 ? {} : args.extract_options! # this is necessary because the old auto_link API has a Hash as its last parameter
-        unless args.empty?
-          options[:link] = args[0] || :all
-          options[:html] = args[1] || {}
-        end
-        options.reverse_merge!(:link => :all, :html => {})
-
-        case options[:link].to_sym
-          when :all                         then auto_link_email_addresses(auto_link_urls(text, options[:html], &block), &block)
-          when :email_addresses             then auto_link_email_addresses(text, &block)
-          when :urls                        then auto_link_urls(text, options[:html], &block)
-        end
+        text.html_safe.safe_concat("</p>")
       end
 
       # Creates a Cycle object whose _to_s_ method cycles through elements of an
       # array every time it is called. This can be used for example, to alternate
-      # classes for table rows.  You can use named cycles to allow nesting in loops.
+      # classes for table rows. You can use named cycles to allow nesting in loops.
       # Passing a Hash as the last parameter with a <tt>:name</tt> key will create a
       # named cycle. The default name for a cycle without a +:name+ key is
       # <tt>"default"</tt>. You can manually reset a cycle by calling reset_cycle
@@ -412,7 +290,7 @@ module ActionView
       #   @items = [1,2,3,4]
       #   <table>
       #   <% @items.each do |item| %>
-      #     <tr class="<%= cycle("even", "odd") -%>">
+      #     <tr class="<%= cycle("odd", "even") -%>">
       #       <td>item</td>
       #     </tr>
       #   <% end %>
@@ -424,7 +302,7 @@ module ActionView
       #                {:first => 'Emily', :middle => 'Shannon', :maiden => 'Pike', :last => 'Hicks'},
       #               {:first => 'June', :middle => 'Dae', :last => 'Jones'}]
       #   <% @items.each do |item| %>
-      #     <tr class="<%= cycle("even", "odd", :name => "row_class") -%>">
+      #     <tr class="<%= cycle("odd", "even", :name => "row_class") -%>">
       #       <td>
       #         <% item.values.each do |value| %>
       #           <%# Create a named cycle "colors" %>
@@ -446,14 +324,14 @@ module ActionView
         values.unshift(first_value)
 
         cycle = get_cycle(name)
-        if (cycle.nil? || cycle.values != values)
+        unless cycle && cycle.values == values
           cycle = set_cycle(name, Cycle.new(*values))
         end
-        return cycle.to_s
+        cycle.to_s
       end
 
       # Returns the current cycle string after a cycle has been started. Useful
-      # for complex table highlighing or any other design need which requires
+      # for complex table highlighting or any other design need which requires
       # the current cycle string in more than one place.
       #
       # ==== Example
@@ -466,7 +344,7 @@ module ActionView
       #   <% end %>
       def current_cycle(name = "default")
         cycle = get_cycle(name)
-        cycle.current_value unless cycle.nil?
+        cycle.current_value if cycle
       end
 
       # Resets a cycle so that it starts from the first element the next time
@@ -490,7 +368,7 @@ module ActionView
       #   </table>
       def reset_cycle(name = "default")
         cycle = get_cycle(name)
-        cycle.reset unless cycle.nil?
+        cycle.reset if cycle
       end
 
       class Cycle #:nodoc:
@@ -543,59 +421,6 @@ module ActionView
           @_cycles = Hash.new unless defined?(@_cycles)
           @_cycles[name] = cycle_object
         end
-
-        AUTO_LINK_RE = %r{
-                        (                          # leading text
-                          <\w+.*?>|                # leading HTML tag, or
-                          [^=!:'"/]|               # leading punctuation, or
-                          ^                        # beginning of line
-                        )
-                        (
-                          (?:https?://)|           # protocol spec, or
-                          (?:www\.)                # www.*
-                        )
-                        (
-                          [-\w]+                   # subdomain or domain
-                          (?:\.[-\w]+)*            # remaining subdomains or domain
-                          (?::\d+)?                # port
-                          (?:/(?:[~\w\+@%=\(\)-]|(?:[,.;:'][^\s$]))*)* # path
-                          (?:\?[\w\+@%&=.;:-]+)?     # query string
-                          (?:\#[\w\-]*)?           # trailing anchor
-                        )
-                        ([[:punct:]]|<|$|)       # trailing text
-                       }x unless const_defined?(:AUTO_LINK_RE)
-
-        # Turns all urls into clickable links.  If a block is given, each url
-        # is yielded and the result is used as the link text.
-        def auto_link_urls(text, html_options = {})
-          extra_options = tag_options(html_options.stringify_keys) || ""
-          text.gsub(AUTO_LINK_RE) do
-            all, a, b, c, d = $&, $1, $2, $3, $4
-            if a =~ /<a\s/i # don't replace URL's that are already linked
-              all
-            else
-              text = b + c
-              text = yield(text) if block_given?
-              %(#{a}<a href="#{b=="www."?"http://www.":b}#{c}"#{extra_options}>#{text}</a>#{d})
-            end
-          end
-        end
-
-        # Turns all email addresses into clickable links.  If a block is given,
-        # each email is yielded and the result is used as the link text.
-        def auto_link_email_addresses(text)
-          body = text.dup
-          text.gsub(/([\w\.!#\$%\-+.]+@[A-Za-z0-9\-]+(\.[A-Za-z0-9\-]+)+)/) do
-            text = $1
-
-            if body.match(/<a\b[^>]*>(.*)(#{Regexp.escape(text)})(.*)<\/a>/)
-              text
-            else
-              display_text = (block_given?) ? yield(text) : text
-              %{<a href="mailto:#{text}">#{display_text}</a>}
-            end
-          end
-        end
     end
   end
 end
