@@ -11,19 +11,14 @@ class ActiveRecordTestConnector
 end
 
 # Try to grab AR
-if defined?(ActiveRecord) && defined?(Fixtures)
-  $stderr.puts 'Active Record is already loaded, running tests'
-else
-  $stderr.print 'Attempting to load Active Record... '
+unless defined?(ActiveRecord) && defined?(Fixtures)
   begin
     PATH_TO_AR = "#{File.dirname(__FILE__)}/../../activerecord/lib"
     raise LoadError, "#{PATH_TO_AR} doesn't exist" unless File.directory?(PATH_TO_AR)
     $LOAD_PATH.unshift PATH_TO_AR
     require 'active_record'
-    require 'active_record/fixtures'
-    $stderr.puts 'success'
   rescue LoadError => e
-    $stderr.print "failed. Skipping Active Record assertion tests: #{e}"
+    $stderr.print "Failed to load Active Record. Skipping Active Record assertion tests: #{e}"
     ActiveRecordTestConnector.able_to_connect = false
   end
 end
@@ -51,7 +46,8 @@ class ActiveRecordTestConnector
         if Object.const_defined?(:ActiveRecord)
           defaults = { :database => ':memory:' }
           begin
-            options = defaults.merge :adapter => 'sqlite3', :timeout => 500
+            adapter = defined?(JRUBY_VERSION) ? 'jdbcsqlite3' : 'sqlite3'
+            options = defaults.merge :adapter => adapter, :timeout => 500
             ActiveRecord::Base.establish_connection(options)
             ActiveRecord::Base.configurations = { 'sqlite3_ar_integration' => options }
             ActiveRecord::Base.connection
@@ -82,7 +78,9 @@ class ActiveRecordTestConnector
   end
 end
 
-class ActiveRecordTestCase < ActiveSupport::TestCase
+class ActiveRecordTestCase < ActionController::TestCase
+  include ActiveRecord::TestFixtures
+
   # Set our fixture path
   if ActiveRecordTestConnector.able_to_connect
     self.fixture_path = [FIXTURE_LOAD_PATH]
@@ -96,8 +94,6 @@ class ActiveRecordTestCase < ActiveSupport::TestCase
   def run(*args)
     super if ActiveRecordTestConnector.connected
   end
-
-  def default_test; end
 end
 
 ActiveRecordTestConnector.setup
