@@ -1,29 +1,36 @@
+# encoding: utf-8
 require 'abstract_unit'
 
-
 module TestFileUtils
   def file_name() File.basename(__FILE__) end
   def file_path() File.expand_path(__FILE__) end
-  def file_data() File.open(file_path, 'rb') { |f| f.read } end
+  def file_data() @data ||= File.open(file_path, 'rb') { |f| f.read } end
 end
 
-
 class SendFileController < ActionController::Base
   include TestFileUtils
   layout "layouts/standard" # to make sure layouts don't interfere
 
   attr_writer :options
-  def options() @options ||= {} end
+  def options
+    @options ||= {}
+  end
 
-  def file() send_file(file_path, options) end
-  def data() send_data(file_data, options) end
+  def file
+    send_file(file_path, options)
+  end
 
-  def rescue_action(e) raise end
+  def data
+    send_data(file_data, options)
+  end
+
+  def multibyte_text_data
+    send_data("Кирилица\n祝您好運.", options)
+  end
 end
 
-SendFileController.view_paths = [ File.dirname(__FILE__) + "/../fixtures/" ]
-
-class SendFileTest < Test::Unit::TestCase
+class SendFileTest < ActionController::TestCase
+  tests SendFileController
   include TestFileUtils
 
   Mime::Type.register "image/png", :png unless defined? Mime::PNG
@@ -39,20 +46,23 @@ class SendFileTest < Test::Unit::TestCase
     response = nil
     assert_nothing_raised { response = process('file') }
     assert_not_nil response
-    assert_kind_of String, response.body
-    assert_equal file_data, response.body
+    body = response.body
+    assert_kind_of String, body
+    assert_equal file_data, body
   end
 
   def test_file_stream
     response = nil
     assert_nothing_raised { response = process('file') }
     assert_not_nil response
-    assert_kind_of Proc, response.body
+    assert_respond_to response.body_parts, :each
+    assert_respond_to response.body_parts, :to_path
 
     require 'stringio'
     output = StringIO.new
     output.binmode
-    assert_nothing_raised { response.body.call(response, output) }
+    output.string.force_encoding(file_data.encoding) if output.string.respond_to?(:force_encoding)
+    assert_nothing_raised { response.body_parts.each { |part| output << part.to_s } }
     assert_equal file_data, output.string
   end
 
@@ -64,17 +74,6 @@ class SendFileTest < Test::Unit::TestCase
     assert_equal "attachment", response.headers["Content-Disposition"]
   end
 
-  def test_x_sendfile_header
-    @controller.options = { :x_sendfile => true }
-
-    response = nil
-    assert_nothing_raised { response = process('file') }
-    assert_not_nil response
-
-    assert_equal @controller.file_path, response.headers['X-Sendfile']
-    assert response.body.blank?
-  end
-
   def test_data
     response = nil
     assert_nothing_raised { response = process('data') }
@@ -86,16 +85,15 @@ class SendFileTest < Test::Unit::TestCase
 
   def test_headers_after_send_shouldnt_include_charset
     response = process('data')
-    assert_equal "application/octet-stream", response.content_type
+    assert_equal "application/octet-stream", response.headers["Content-Type"]
 
     response = process('file')
-    assert_equal "application/octet-stream", response.content_type
+    assert_equal "application/octet-stream", response.headers["Content-Type"]
   end
 
   # Test that send_file_headers! is setting the correct HTTP headers.
-  def test_send_file_headers!
+  def test_send_file_headers_bang
     options = {
-      :length => 1,
       :type => Mime::PNG,
       :disposition => 'disposition',
       :filename => 'filename'
@@ -110,29 +108,85 @@ class SendFileTest < Test::Unit::TestCase
     @controller.send(:send_file_headers!, options)
 
     h = @controller.headers
-    assert_equal 1, h['Content-Length']
-    assert_equal 'image/png', h['Content-Type']
+    assert_equal 'image/png', @controller.content_type
     assert_equal 'disposition; filename="filename"', h['Content-Disposition']
     assert_equal 'binary', h['Content-Transfer-Encoding']
 
     # test overriding Cache-Control: no-cache header to fix IE open/save dialog
-    @controller.headers = { 'Cache-Control' => 'no-cache' }
     @controller.send(:send_file_headers!, options)
-    h = @controller.headers
+    @controller.response.prepare!
     assert_equal 'private', h['Cache-Control']
   end
 
+  def test_send_file_headers_with_disposition_as_a_symbol
+    options = {
+      :type => Mime::PNG,
+      :disposition => :disposition,
+      :filename => 'filename'
+    }
+
+    @controller.headers = {}
+    @controller.send(:send_file_headers!, options)
+    assert_equal 'disposition; filename="filename"', @controller.headers['Content-Disposition']
+  end
+
+  def test_send_file_headers_with_mime_lookup_with_symbol
+    options = {
+      :type => :png
+    }
+
+    @controller.headers = {}
+    @controller.send(:send_file_headers!, options)
+
+    assert_equal 'image/png', @controller.content_type
+  end
+
+
+  def test_send_file_headers_with_bad_symbol
+    options = {
+      :type => :this_type_is_not_registered
+    }
+
+    @controller.headers = {}
+    assert_raise(ArgumentError){ @controller.send(:send_file_headers!, options) }
+  end
+
+  def test_send_file_headers_guess_type_from_extension
+    {
+      'image.png' => 'image/png',
+      'image.jpeg' => 'image/jpeg',
+      'image.jpg' => 'image/jpeg',
+      'image.tif' => 'image/tiff',
+      'image.gif' => 'image/gif',
+      'movie.mpg' => 'video/mpeg',
+      'file.zip' => 'application/zip',
+      'file.unk' => 'application/octet-stream',
+      'zip' => 'application/octet-stream'
+    }.each do |filename,expected_type|
+      options = { :filename => filename }
+      @controller.headers = {}
+      @controller.send(:send_file_headers!, options)
+      assert_equal expected_type, @controller.content_type
+    end
+  end
+
   %w(file data).each do |method|
     define_method "test_send_#{method}_status" do
       @controller.options = { :stream => false, :status => 500 }
       assert_nothing_raised { assert_not_nil process(method) }
-      assert_equal '500 Internal Server Error', @response.headers['Status']
+      assert_equal 500, @response.status
+    end
+
+    define_method "test_send_#{method}_content_type" do
+      @controller.options = { :stream => false, :content_type => "application/x-ruby" }
+      assert_nothing_raised { assert_not_nil process(method) }
+      assert_equal "application/x-ruby", @response.content_type
     end
 
     define_method "test_default_send_#{method}_status" do
       @controller.options = { :stream => false }
       assert_nothing_raised { assert_not_nil process(method) }
-      assert_equal ActionController::Base::DEFAULT_RENDER_STATUS_CODE, @response.headers['Status']
+      assert_equal 200, @response.status
     end
   end
 end
