@@ -1,6 +1,13 @@
 require 'abstract_unit'
 
-Scroll = Struct.new(:id, :to_param, :title, :body, :updated_at, :created_at)
+class Scroll < Struct.new(:id, :to_param, :title, :body, :updated_at, :created_at)
+  extend ActiveModel::Naming
+  include ActiveModel::Conversion
+
+  def persisted?
+    false
+  end
+end
 
 class ScrollsController < ActionController::Base
   FEEDS = {}
@@ -9,7 +16,7 @@ class ScrollsController < ActionController::Base
           feed.title("My great blog!")
           feed.updated((@scrolls.first.created_at))
 
-          for scroll in @scrolls
+          @scrolls.each do |scroll| 
             feed.entry(scroll) do |entry|
               entry.title(scroll.title)
               entry.content(scroll.body, :type => 'html')
@@ -26,8 +33,8 @@ class ScrollsController < ActionController::Base
           feed.title("My great blog!")
           feed.updated((@scrolls.first.created_at))
 
-          for scroll in @scrolls
-            feed.entry(scroll, :url => "/otherstuff/" + scroll.to_param, :updated => Time.utc(2007, 1, scroll.id)) do |entry|
+          @scrolls.each do |scroll|
+            feed.entry(scroll, :url => "/otherstuff/" + scroll.to_param.to_s, :updated => Time.utc(2007, 1, scroll.id)) do |entry|
               entry.title(scroll.title)
               entry.content(scroll.body, :type => 'html')
 
@@ -47,8 +54,8 @@ class ScrollsController < ActionController::Base
             author.name("DHH")
           end
 
-          for scroll in @scrolls
-            feed.entry(scroll, :url => "/otherstuff/" + scroll.to_param, :updated => Time.utc(2007, 1, scroll.id)) do |entry|
+          @scrolls.each do |scroll|
+            feed.entry(scroll, :url => "/otherstuff/" + scroll.to_param.to_s, :updated => Time.utc(2007, 1, scroll.id)) do |entry|
               entry.title(scroll.title)
               entry.content(scroll.body, :type => 'html')
             end
@@ -61,7 +68,7 @@ class ScrollsController < ActionController::Base
           feed.title("My great blog!")
           feed.updated((@scrolls.first.created_at))
 
-          for scroll in @scrolls
+          @scrolls.each do |scroll|
             feed.entry(scroll) do |entry|
               entry.title(scroll.title)
               entry.content(scroll.body, :type => 'html')
@@ -79,7 +86,7 @@ class ScrollsController < ActionController::Base
           feed.title("My great blog!")
           feed.updated((@scrolls.first.created_at))
 
-          for scroll in @scrolls
+          @scrolls.each do |scroll|
             feed.entry(scroll, :id => "tag:test.rubyonrails.org,2008:"+scroll.id.to_s) do |entry|
               entry.title(scroll.title)
               entry.content(scroll.body, :type => 'html')
@@ -98,7 +105,7 @@ class ScrollsController < ActionController::Base
           feed.title("My great blog!")
           feed.updated((@scrolls.first.created_at))
 
-          for scroll in @scrolls
+          @scrolls.each do |scroll|
             feed.entry(scroll) do |entry|
               entry.title(scroll.title)
               entry.content(scroll.body, :type => 'html')
@@ -116,7 +123,7 @@ class ScrollsController < ActionController::Base
           feed.title("My great blog!")
           feed.updated((@scrolls.first.created_at))
 
-          for scroll in @scrolls
+          @scrolls.each do |scroll|
             feed.entry(scroll) do |entry|
               entry.title(scroll.title)
               entry.content(scroll.body, :type => 'html')
@@ -133,7 +140,7 @@ class ScrollsController < ActionController::Base
           feed.title("My great blog!")
           feed.updated((@scrolls.first.created_at))
 
-          for scroll in @scrolls
+          @scrolls.each do |scroll|
             feed.entry(scroll) do |entry|
               entry.title(scroll.title)
               entry.summary(:type => 'xhtml') do |xhtml|
@@ -158,7 +165,7 @@ class ScrollsController < ActionController::Base
             feed.title("My great blog!")
             feed.updated((@scrolls.first.created_at))
 
-            for scroll in @scrolls
+            @scrolls.each do |scroll|
               feed.entry(scroll) do |entry|
                 entry.title(scroll.title)
                 entry.content(scroll.body, :type => 'html')
@@ -180,23 +187,23 @@ class ScrollsController < ActionController::Base
   end
 
   protected
-
-  def rescue_action(e)
-    raise(e)
-  end
+    def rescue_action(e)
+      raise(e)
+    end
 end
 
 class AtomFeedTest < ActionController::TestCase
   tests ScrollsController
 
   def setup
+    super
     @request.host = "www.nextangle.com"
   end
 
   def test_feed_should_use_default_language_if_none_is_given
     with_restful_routing(:scrolls) do
       get :index, :id => "defaults"
-      assert_match %r{xml:lang="en-US"}, @response.body
+      assert_match(%r{xml:lang="en-US"}, @response.body)
     end
   end
 
@@ -219,7 +226,7 @@ class AtomFeedTest < ActionController::TestCase
       get :index, :id=>"provide_builder"
       # because we pass in the non-default builder, the content generated by the
       # helper should go 'nowhere'.  Leaving the response body blank.
-      assert @response.body.blank?
+      assert_blank @response.body
     end
   end
 
@@ -303,11 +310,12 @@ class AtomFeedTest < ActionController::TestCase
       assert_select "summary div p", :text => "after 2"
     end
   end
-private
+
+  private
     def with_restful_routing(resources)
       with_routing do |set|
-        set.draw do |map|
-          map.resources(resources)
+        set.draw do
+          resources(resources)
         end
         yield
       end
